package landlordApp;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * -Tietää asukkaan kentät                        
 * -Osaa tarkistaa kenttien oikeellisuuden(Syntax)
 * -Osaa muuttaa asukkaan merkkijonon asukkaan    
 * tiedoiksi                                     
 * -Osaa antaa merkkijonona i:n kentän tiedot     
 * -Osaa laittaa merkkijonon i:neksi kentäksi                                                     
 * @author Nico & Eelis
 * @version 12.3.2020
 */
public class Asukas {
	private int id;
	private int asuntoId;
	
	private String etunimet;
	private String sukunimi;
	private String puhNro;
	private String sPosti;
	private int ika;
	
	private static Random rand;
	private static int seuraavaNro = 1;
	
	
	/**
	 * Rekisteroi uuden talon ja kasvattaa talojen lukumäärää yhdellä.
	 * @return id
	 * @example
	 * <pre name="test">
	 * Asukas asukas = new Asukas(1);
	 * asukas.getId() === 0;
	 * asukas.rekisteroi();
	 * Asukas asukas2 = new Asukas();
	 * asukas2.rekisteroi();
	 * int n1 = asukas.getId();
	 * int n2 = asukas2.getId();
	 * n1 === n2-1;
	 * </pre>
	 */
	public int rekisteroi() {
		this.id = seuraavaNro;
		Asukas.seuraavaNro++;
		return id;
	}
	
	
	/**
	 * Alustaa uuden asukkaan tiettyyn asuntoon.
	 * @param asuntoId asunnon id
	 */
	public Asukas(int asuntoId) {
		this.asuntoId = asuntoId;
	}
	
	/**
	 * Alustaa uuden tyhjän asukkaan.
	 */
	public Asukas() {
		
	}
	
	
	/**
	 * @return asukkaan id
	 */
	public int getId() {
		return this.id;
	}
	
	/**
	 * @return asukkaan asuntoId
	 */
	public int getAsuntoId() {
		return this.asuntoId;
	}
	
	public String getAsukaanEtunimi() {
	    return this.etunimet;
	}
	
	/**
	 * Täyttää asukkaan tiedot oletustiedoilla testaamista varten.
	 */
	public void taytaOletus() {
		rand = new Random();
		int randomNro = rand.nextInt(9999999);
		ArrayList<String> randEtunimet = new ArrayList<String>();
		randEtunimet.add("Kallu");
		randEtunimet.add("Matti");
		randEtunimet.add("Teppo");
		randEtunimet.add("Seppo");
		randEtunimet.add("Eetu");
		randEtunimet.add("king_elkku007");
		
		ArrayList<String> randSukunimet = new ArrayList<String>();
		randSukunimet.add("Meikäläinen");
		randSukunimet.add("Kallunen");
		randSukunimet.add("Kääriäinen");
		randSukunimet.add("Mikkonen");
		randSukunimet.add("Ukkonen");
		randSukunimet.add("THESLAYER");
		
		etunimet = randEtunimet.get(rand.nextInt(randEtunimet.size()));
		sukunimi = randSukunimet.get(rand.nextInt(randSukunimet.size()));
		puhNro =  "045 " + String.format("%07d", randomNro);
		sPosti = (etunimet.toLowerCase() + "." + sukunimi.toLowerCase()  + "@gmail.com").replace('ä', 'a').replace('ö', 'o');
		ika = 18 + rand.nextInt(40);
		
	}
	
	
	/**
	 * Metodi tekee asukkaan tiedoista merkkijonon, joka on helposti tallennettavissa esim. tiedostoon
	 */
	@Override
    public String toString() {
	    return "" + getId() +"|" +
	            getAsuntoId() + "|" +
	            etunimet + "|" +
	            sukunimi + "|" +
	            puhNro + "|" +
	            sPosti +  "|" +
	            ika;
	            
	}
	
	
	/**
	 * Ottaa syötetystä merkkijonosta tiedot ja asettaa ne asukkaalle
	 * @param rivi syötetty merkkijono ( yleensä tiedostosta)
	 * @example
	 * <pre name="test">
	 * Asukas kallu = new Asukas();
	 * kallu.parse("1 | 2 | Kallu | Kallunen | 0400841251 | kallu.kallunen@gmail.com | 20");
	 * kallu.getId() === 1;
	 * kallu.toString().equals("1|2|Kallu|Kallunen|0400841251|kallu.kallunen@gmail.com|20"); 
	 * </pre>
	 */
	public void parse(String rivi) {
	    StringBuffer sb = new StringBuffer(rivi);
	    id = Mjonot.erota(sb, '|', getId());
	    asuntoId = Mjonot.erota(sb, '|', getAsuntoId());
	    etunimet = Mjonot.erota(sb, '|', etunimet);
	    sukunimi = Mjonot.erota(sb, '|', sukunimi);
	    puhNro = Mjonot.erota(sb, '|', puhNro);
	    sPosti = Mjonot.erota(sb, '|',sPosti);
	    ika = Mjonot.erota(sb, '|', ika);
	}
	
	/**
	 * Tulostaa asukkaan tiedot tietovirtaan
	 * @param os tietovirta, johon tulostetaan.
	 */
	public void tulosta(PrintStream os) {
		os.println("Id : " + this.id);
		os.println("AsuntoId: " + this.asuntoId);
		os.println(etunimet + " " + sukunimi);
		os.println(puhNro);
		os.println(sPosti);
		os.println("Ikä: " + ika + "v" + "\n");
	}
	
	
	/**
	 * Tulostaa asukkaan tiedot tietovirtaan, tyypiltään outputstream
	 * @param os tietovirta johon tulostetaan
	 */
	public void tulosta(OutputStream os) {
		tulosta(new PrintStream(os));
	}
	
	
	/**
	 * @param args ei käytössä
	 */
	public static void main(String[] args) {
		
		Asukas kallu = new Asukas(1);
		kallu.rekisteroi();
		kallu.taytaOletus();
		kallu.tulosta(System.out);
		kallu.parse("2|2|Eelis|Koivusaari|0405219330|eelis.koivusaari@gmail.com|19");
		kallu.tulosta(System.out);
	}

}
