package landlordApp;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;

/**
 * -Pitää yllä asukkaiden rekisteriä, osaa lisätä ja
 * poistaa asukkaan                                
 * -Lukee ja kirjoittaa asukkaat tiedostoon         
 * -Osaa etsiä ja lajitella                         
 * @author Nico & Eelis
 * @version 12.3.2020
 *
 */
public class Asukkaat implements Iterable<Asukas>{
	 private boolean muutettu = false;
	 private String tiedostonPerusNimi = "asukkaat";
	 private final Collection<Asukas> asukkaat = new ArrayList<Asukas>();
	 
	 
	 /**
	  * @return asukkaiden lukumäärä
	  */
	 public int getLkm() {
		 return asukkaat.size();
	 }
	 
	 /**
	  * Lisää asukkaiden listaan uuden asukkaan.
	  * @param asukas lisättävä asukas
	 * @throws SailoException testaa voidanko lisätä
	  * @example
	  * <pre name="test">
	  * Asukkaat asukkaat = new Asukkaat();
	  * Asukas asukas1 = new Asukas(1); Asukas asukas2 = new Asukas(1);
	  * asukkaat.getLkm() === 0;
	  * asukkaat.lisaa(asukas1); asukkaat.getLkm() === 1;
	  * asukkaat.lisaa(asukas2); asukkaat.getLkm() === 2;
	  * asukkaat.lisaa(asukas2); asukkaat.getLkm() === 3;
	  * asukkaat.anna(0) === asukas1;
	  * asukkaat.anna(1) === asukas2;
	  * asukkaat.anna(2) === asukas2;
	  * asukkaat.anna(1) == asukas1 === false;
	  * asukkaat.anna(1) == asukas2 === true;
	  * asukkaat.anna(3) === asukas1; #THROWS IndexOutOfBoundsException
	  * asukkaat.lisaa(asukas1); asukkaat.getLkm() === 4;
	  * asukkaat.lisaa(asukas1); asukkaat.getLkm() === 5;
	  * asukkaat.lisaa(asukas1);
	  * </pre>
	  */
	 public void lisaa(Asukas asukas) {
		 this.asukkaat.add(asukas);
		 muutettu = true;
	 }
	 
	 /**
	  * Palauttaa paikassa i olevan asukkaan viitteen
	  * @param i indeksi arraylistissä
	  * @return etsitty asukas
	  * @throws IndexOutOfBoundsException jos indeksi on virheellinen, eli liian suuri tai pieni.
	  */
	 public Asukas anna(int i) throws IndexOutOfBoundsException {
		 if(i < 0 || asukkaat.size() <= i)
	            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
	        return ((ArrayList<Asukas>) asukkaat).get(i);
	 }
	 
	 /**
	  * @return tiedoston nimi ilman tarkenninta
	  */
	 public String getTiedostonPerusNimi() {
		 return tiedostonPerusNimi;
	 }
	 
	 /**
	  * @return tiedoston nimi
	  */
	 public String getTiedostonNimi() {
		 return tiedostonPerusNimi + ".dat";
	 }
	 
	 /**
	  * Asettaa tiedostolle nimen
	  * @param tied tiedostonnimi
	  */
	 public void setTiedostonPerusNimi(String tied) {
		 this.tiedostonPerusNimi = tied;
	 }
	 
	 /**
	  * @return varmuuskopion nimi
	  */
	 public String getBakNimi() {
		 return tiedostonPerusNimi + ".bak";
	 }
	 
	 /**
	     * Iteraattori kaikkien asukkaiden läpikäymiseen
	     * @return asukasiteraattori
	     * 
	     * @example
	     * <pre name="test">
	     * #PACKAGEIMPORT
	     * #import java.util.*;
	     * 
	     *  Asukkaat asukkaat = new Asukkaat();
	     *  Asukas pitsi21 = new Asukas(2); asukkaat.lisaa(pitsi21);
	     *  Asukas pitsi11 = new Asukas(1); asukkaat.lisaa(pitsi11);
	     *  Asukas pitsi22 = new Asukas(2); asukkaat.lisaa(pitsi22);
	     *  Asukas pitsi12 = new Asukas(1); asukkaat.lisaa(pitsi12);
	     *  Asukas pitsi23 = new Asukas(2); asukkaat.lisaa(pitsi23);
	     * 
	     *  Iterator<Asukas> i2=asukkaat.iterator();
	     *  i2.next() === pitsi21;
	     *  i2.next() === pitsi11;
	     *  i2.next() === pitsi22;
	     *  i2.next() === pitsi12;
	     *  i2.next() === pitsi23;
	     *  i2.next() === pitsi12;  #THROWS NoSuchElementException  
	     *  
	     *  int n = 0;
	     *  int jnrot[] = {2,1,2,1,2};
	     *  
	     *  for ( Asukas as:asukkaat ) { 
	     *    as.getAsuntoId() === jnrot[n]; n++;  
	     *  }
	     *  
	     *  n === 5;
	     *  
	     * </pre>
	     */
	 @Override
	 public Iterator<Asukas> iterator(){
		 return asukkaat.iterator();
	 }
	 
	 /**
	  * Palauttaa listan asukkaista, joilla on täsmäävä asuntoId.
	  * @param id asunnon id
	  * @return lista löydetyistä asukkaista
	  * @example
	  * #import java.util.List;
	  * <pre name="test">
	  * Asukkaat asukkaat = new Asukkaat();
	  * Asukas asukas1 = new Asukas(1);
	  * Asukas asukas2 = new Asukas(2);
	  * Asukas asukas3 = new Asukas(1);
	  * asukkaat.lisaa(asukas1);
	  * asukkaat.lisaa(asukas2);
	  * asukkaat.lisaa(asukas3);
	  * List<Asukas> loydetyt1 = asukkaat.annaAsukkaat(1);
	  * loydetyt1.size() === 2;
	  * List<Asukas> loydetyt2 = asukkaat.annaAsukkaat(2);
	  * loydetyt2.size() === 1;
	  * loydetyt1.get(0) === asukas1;
	  * </pre>
	  */
	 public List<Asukas> annaAsukkaat(int id){
	        List<Asukas> loydetyt = new ArrayList<Asukas>();
	        for (Asukas as : asukkaat)
	            if(as.getAsuntoId() == id) loydetyt.add(as);
	        
	        return loydetyt;
	        
	    }
	 
	 /**
	  * Lukee tiedostosta asukkaiden tiedot ja luo oliot niiden perusteella.
	  * @param tied luettavan tiedoston nimi
	  * @throws SailoException jos tulee ongelmia
	  * @example
      * <pre name="test">
      * #THROWS SailoException 
      * #import java.io.File;
      * #import java.util.Iterator;
      *  Asukkaat asukkaat = new Asukkaat();
      *  Asukas pitsi21 = new Asukas(); pitsi21.taytaOletus();
      *  Asukas pitsi11 = new Asukas(); pitsi11.taytaOletus();
      *  Asukas pitsi22 = new Asukas(); pitsi22.taytaOletus(); 
      *  Asukas pitsi12 = new Asukas(); pitsi12.taytaOletus();
      *  Asukas pitsi23 = new Asukas(); pitsi23.taytaOletus(); 
      *  String tiedNimi = "testiasukkaat";
      *  File ftied = new File(tiedNimi+".dat");
      *  ftied.delete();
      *  asukkaat.lueTiedostosta(tiedNimi); #THROWS SailoException
      *  asukkaat.lisaa(pitsi21);
      *  asukkaat.lisaa(pitsi11);
      *  asukkaat.lisaa(pitsi22);
      *  asukkaat.lisaa(pitsi12);
      *  asukkaat.lisaa(pitsi23);
      *  asukkaat.tallenna();
      *  asukkaat = new Asukkaat();
      *  asukkaat.lueTiedostosta(tiedNimi);
      *  Iterator<Asukas> i = asukkaat.iterator();
      *  i.next().toString() === pitsi21.toString();
      *  i.next().toString() === pitsi11.toString();
      *  i.next().toString() === pitsi22.toString();
      *  i.next().toString() === pitsi12.toString();
      *  i.next().toString() === pitsi23.toString();
      *  i.hasNext() === false;
      *  asukkaat.lisaa(pitsi23);
      *  asukkaat.tallenna();
      *  ftied.delete() === true;
      *  File fbak = new File(tiedNimi+".bak");
      *  fbak.delete() === true;
      * </pre>
      */
	 public void lueTiedostosta(String tied) throws SailoException {
		setTiedostonPerusNimi(tied);
		 String rivi;
		 try (Scanner fi = new Scanner(new FileInputStream(new File(getTiedostonNimi())))){
			 while ( fi.hasNext()) {
				 rivi = fi.nextLine().trim();
				 if ("".equals(rivi) || rivi.charAt(0) == ';') continue;
				 Asukas asukas = new Asukas();
				 asukas.parse(rivi);
				 lisaa(asukas);
			 }
			 muutettu = false;
		}catch (FileNotFoundException e) {
			throw new SailoException("Tiedosto " + getTiedostonNimi() + " ei aukea");
		}
	 }
	 
	 /**
	  * Lukee tiedoston oletusnimellä
	  * @throws SailoException jos tulee poikkeus
	  */
	 public void lueTiedostosta() throws SailoException{
		 lueTiedostosta(getTiedostonPerusNimi());
	 }
	 
	 /**
	  * Tallettaa tiedostoon tämänhetkiset oliot.
	  * @throws SailoException jos tulee ongelmia
	  */
	 public void tallenna() throws SailoException{
		 if (!muutettu) return;
		 
		 File fbak = new File(getBakNimi());
		 File ftied = new File(getTiedostonNimi());
		 fbak.delete();
		 ftied.renameTo(fbak);
		 
		 try(PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath()))){
			 for ( Asukas asukas : this) {
				 fo.println(asukas.toString());
			 }
		 }catch (FileNotFoundException e) {
			 throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
			 
		 }catch (IOException e) {
			 throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
		 }
		 muutettu = false;
	 }
	 
	 
	 /**
	  * @param args ei käytössä
	  */
	public static void main(String[] args) {
		Asukkaat asukkaat = new Asukkaat();
		
		Asukas testi1 = new Asukas(1);
		testi1.taytaOletus();
		Asukas testi2 = new Asukas(1);
		testi2.taytaOletus();
		
		asukkaat.lisaa(testi1);
		asukkaat.lisaa(testi2);
		
		try {
			asukkaat.tallenna();
		}catch (SailoException e) {
			System.err.println(e.getMessage());
		}
		
		try {
			asukkaat.lueTiedostosta();
		}catch(SailoException e) {
			System.err.println(e.getMessage());
		}
		
		
		System.out.println("=============== Asukkaat testi ===============");
		
		for (Asukas as : asukkaat) {
			

			as.tulosta(System.out);
		}
		
	}
}
