package landlordApp;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * Vastuualueet:
 * -Tietää asunnolle vaaditut kentät
 * -Osaa tarkastaa tietyn kentän oikeellisuuden
 *  eli syntakstin.
 * -Osaa muuttaa asunto tiedot tiedostosta asunnon
 *  tiedoiksi
 * -Osaa antaa merkkijonona i:n kentän tiedot
 * -Osaa laittaa merkkijonon i:neksi kentäksi
 * @author Nico & Eelis
 * @version 12.3.2020
 *
 */
public class Asunto {
    //Attribuutit
    private int id;
    private int taloid;
    private String asuntoid;
    private int neliot;
    private String viimKorjaus;
    private double vuokra;
    private String lisatietoja;
    
    private static Random rand;
    private static int seuraavaNro = 1;
    
    /**
     * Alustaa uuden asunnon tietylle taloid:lle
     * @param taloId talon id, johon asunto lisätään
     */
    public Asunto(int taloId) {
        this.taloid = taloId;
    }
    
    /**
     * Alustaa tyhjän asunto-olion.
     */
    public Asunto() {
    }
    
    /**
     * Rekisteröi uuden asunnon tietorakenteeseen.
     * @return rekisteröidyn asunnon id.
     * @example
	 * <pre name="test">
	 * Asunto asunto = new Asunto();
	 * asunto.getId() === 0;
	 * asunto.rekisteroi();
	 * Asunto asunto2 = new Asunto();
	 * asunto2.rekisteroi();
	 * int n1 = asunto.getId();
	 * int n2 = asunto2.getId();
	 * n1 === n2-1;
	 * </pre>
     */
    public int rekisteroi() {
        this.id = seuraavaNro;
        Asunto.seuraavaNro++;
        return id;
        
    }
    
    /**
     * @return asunnon taloid
     */
    public int getTaloId() {
    	return this.taloid;
    }
    
    
    /**
     * Palauttaa asunnon id:n
     * @return asunnon id
     */
    public int getId() {
        return this.id;
    }
    
    /**
     * Palauttaa asunnon nimen, eli asuntoid:n
     * @return asunnon id
     */
    public String getAsuntoId() {
       return this.asuntoid;
    }
    
    /**
     * Täyttää asunnon tiedot oletustiedoilla testaamista varten.
     */
    public void taytaOletus() {
        rand = new Random();
        
        asuntoid = "A"+rand.nextInt(10);
        vuokra = 300 + rand.nextInt(300);
        neliot = 20 + rand.nextInt(40);
        viimKorjaus = String.format("%02d.%02d.%4d", rand.nextInt(28), rand.nextInt(12), 1950 + rand.nextInt(70));
        lisatietoja = "";
    }
    
    
    @Override
    public String toString() {
        return "" + getId() +"|" +
                taloid + "|" +
                getAsuntoId() + "|" +
                neliot + "|" +
                viimKorjaus + "|" +
                vuokra +  "|" +
                lisatietoja;
                
    }
    
    
    /**
     * Ottaa syötetystä merkkijonosta tiedot ja asettaa ne asunnolle
     * @param rivi syötetty merkkijono ( yleensä tiedostosta)
     * @example
     * <pre name="test">
     * Asunto ab = new Asunto();
     * ab.parse("3|2|A3|28|10.04.1983|389.0|");
     * ab.getId() === 3;
     * ab.toString().equals("3|2|A3|28|10.04.1983|389.0|"); 
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        id = Mjonot.erota(sb, '|', getId());
        taloid = Mjonot.erota(sb, '|', taloid);
        asuntoid = Mjonot.erota(sb, '|', getAsuntoId());
        neliot = Mjonot.erota(sb, '|', neliot);
        viimKorjaus = Mjonot.erota(sb, '|', viimKorjaus);
        vuokra = Mjonot.erota(sb, '|',vuokra);
        lisatietoja = Mjonot.erota(sb, '|', lisatietoja);
    }
    /**
     * Tulostaa talon tiedot.
     * @param out tietovirta johon tulostetaan.
     */
    public void tulosta(PrintStream out) {
        out.println(id);
        out.println("Talo id: " + taloid);
        out.println(asuntoid);
        out.println("Neliöt: " + neliot);
        out.println("Viimeisin korjaus: " + viimKorjaus);
        out.println(vuokra + "€");
        out.println(lisatietoja + "\n");
        
    }
    
    /**
     * Tulostetaan talon tiedot
     * @param os tietovirta johon tulostetaan.
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Asunto koti = new Asunto(1);
        Asunto koti2 = new Asunto(1);
        
        koti.rekisteroi();
        koti2.rekisteroi();
        
        koti.taytaOletus();
        koti2.taytaOletus();
        
        koti.tulosta(System.out);
        koti2.tulosta(System.out);
        
    }
}
